@@ -71,7 +71,7 @@
 #include <netinet/in.h>	/* Consts & structs defined by the internet system */
 
 /* good old times without autoconf... */
-#if defined(linux) || defined(sun)
+#if defined(__linux__) || defined(__sun__) || defined(__CYGWIN__)
 #include <sys/sysmacros.h>
 #endif
 
@@ -214,12 +214,13 @@ int findalign(struct filenode *node)
 
 int romfs_checksum(void *data, int size)
 {
-        int32_t sum, *ptr;
+        int32_t sum, word, *ptr;
 
         sum = 0; ptr = data;
         size>>=2;
         while (size>0) {
-                sum += ntohl(*ptr++);
+                word = *ptr++;
+                sum += ntohl(word);
                 size--;
         }
         return sum;
@@ -557,7 +558,10 @@ int processdir(int level, const char *base, const char *dirname, struct stat *sb
 	}
 
 	dirfd = opendir(dir->realname);
-	while((dp = readdir(dirfd))) {
+	if (dirfd == NULL) {
+		perror(dir->realname);
+	}
+	while(dirfd && (dp = readdir(dirfd))) {
 		/* don't process main . and .. twice */
 		if (level <= 1 &&
 		    (strcmp(dp->d_name, ".") == 0
@@ -599,7 +603,7 @@ int processdir(int level, const char *base, const char *dirname, struct stat *sb
 				int       major;
 				int       minor;
 						
-				if (sscanf(n->name, "@%[a-zA-Z0-9],%c,%d,%d",
+				if (sscanf(n->name, "@%[a-zA-Z0-9_+-],%c,%d,%d",
 					   devname, &type, &major, &minor) == 4 ) {
 					strcpy(n->name, devname);
 					sb->st_rdev = makedev(major, minor);
@@ -672,7 +676,7 @@ int processdir(int level, const char *base, const char *dirname, struct stat *sb
 			}
 		}
 	}
-	closedir(dirfd);
+	if (dirfd) closedir(dirfd);
 	return curroffset;
 }
 
